import { getUID } from '../utils';

class WSBaseHandlers {
    updateConnection = (params) => {
        // console.info('[BaseHandlers] update ws connection');
        
        Object.keys(params).forEach((param) => {
            this[param] = params[param];
        });

        this.init();
    };

    subscribe = (channels) => {
        // console.info('[BaseHandlers] subscribe to ', channels);

        this.openedChannels = [...this.openedChannels, ...channels];

        channels.forEach((channel) => {
            const message = {
                jsonrpc: '2.0',
                method: 'add',
                params: {
                    entity: 'subscriptions',
                    data: [channel],
                },
            };

            this.safeSend(message);
        });
    };

    unsubscribe = (channels) => {
        // console.info('[BaseHandlers] unsubscribe from ', channels);

        this.openedChannels = this.openedChannels.filter((openedChannel) => !channels.find((channel) => openedChannel === channel)); // check it

        channels.forEach((channel) => {
            const message = {
                jsonrpc: '2.0',
                method: 'delete',
                params: {
                    entity: 'subscriptions',
                    data: [channel],
                },
            };

            this.safeSend(message);
        });
    };

    safeSend = (message) => {
        const { ws } = this;

        const formattedMessage = {
            ...message,
            id: getUID(),
            params: {
                ...message.params,
                project_uid: this.projectUid,
                userEmail: this.userEmail,
                authToken: this.userToken,
            },
        };

        if (ws && ws.readyState) {
            try {
                ws.send(JSON.stringify(formattedMessage));
                // console.info('[BaseHandlers] send to server', formattedMessage);
            } catch (e) {
                console.error('error sending msg: %o, error: %o', formattedMessage, e);
            }
        } else {
            setTimeout(this.safeSend, 500, formattedMessage);
        }
    };
}

export default WSBaseHandlers;
